@@ -7,10 +7,9 @@ try:
 
     # Load environment variables from .env file
     load_dotenv()
-
-    # Default to development mode if not set
+    # Default to production mode if not set
     IS_DEV = os.getenv("WIDGET_ENV", "production").lower() == "development"
-
+    
 except ImportError:
     IS_DEV = False
 
@@ -18,15 +17,16 @@ except ImportError:
 STATIC_DIR = pathlib.Path(__file__).parent.parent / "static"
 
 if IS_DEV:
-    ROOT_DIR = pathlib.Path(__file__).parent.parent.parent.parent.parent
+    ROOT_DIR = pathlib.Path(__file__).parent.parent.parent.parent.parent.parent
     CSS = open(ROOT_DIR / "js" / "src" / "css" / "styles.css", "r").read()
 
     # Development server configuration
     DEV_SERVER = os.getenv("VITE_DEV_SERVER", "http://localhost:5173")
-    DEV_COMPONENT_PATH = f"{DEV_SERVER}/src/components/widgets"
+    DEV_COMPONENT_PATH = f"{DEV_SERVER}/components/widgets"
 
+    print(f"RUNNING NUMEROUS WIDGETS IN DEVELOPMENT MODE\n\nPlease ensure dev server running on {DEV_SERVER} using 'npx vite'\n")
 else:
-    CSS = STATIC_DIR / "style.css"
+    CSS = open(STATIC_DIR / "styles.css", "r").read()
 
 def get_widget_paths(component_name: str) -> tuple[Union[str, pathlib.Path], pathlib.Path]:
     """
@@ -45,6 +45,6 @@ def get_widget_paths(component_name: str) -> tuple[Union[str, pathlib.Path], pat
 
     else:
         esm = STATIC_DIR / f"{component_name}.mjs"
-        css = STATIC_DIR / f"{component_name}.css"
+        css = CSS
     
     return esm, css
\ No newline at end of file
